@@ -1,3 +1,27 @@
+/*
+ * Dropbear - a SSH2 server
+ * 
+ * Copyright (c) 2002,2003 Matt Johnston
+ * All rights reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following condition:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE. */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -41,8 +65,8 @@ void write_packet() {
 	assert(len > 0);
 	written = write(ses.sock, buf_getptr(writebuf, len), len);
 
-	if (written <= 0) {
-		if (errno == EINTR) { /* TODO eof */
+	if (written < 0) {
+		if (errno == EINTR) {
 			TRACE(("leave writepacket: EINTR"));
 			return;
 		} else {
@@ -50,6 +74,10 @@ void write_packet() {
 		}
 	} 
 
+	if (written == 0) {
+		dropbear_close("remote host closed connection");
+	}
+
 	if (written == len) {
 		dequeue(&ses.writequeue);
 		buf_free(writebuf);
@@ -88,7 +116,7 @@ void read_packet() {
 	buf_incrpos(ses.readbuf, len);
 
 	if (len == 0) {
-		dropbear_close("Connection closed");
+		dropbear_close("remote host closed connection");
 	}
 
 	if (len < 0) {
@@ -133,7 +161,7 @@ static void read_packet_init() {
 	len = read(ses.sock, buf_getwriteptr(ses.readbuf, maxlen),
 			maxlen);
 	if (len == 0) {
-		dropbear_close("Connection closed");
+		dropbear_close("remote host closed connection");
 	}
 	if (len < 0) {
 		dropbear_exit("error reading");
@@ -366,17 +394,15 @@ void process_packet() {
 		case SSH_MSG_IGNORE:
 		case SSH_MSG_DEBUG:
 			TRACE(("received SSH_MSG_IGNORE or SSH_MSG_DEBUG"));
-			/* eat the packet and return */
 			goto out;
 
 		case SSH_MSG_UNIMPLEMENTED:
 			/* debugging XXX */
 			TRACE(("SSH_MSG_UNIMPLEMENTED"));
-			exit(1);
-
+			dropbear_exit("received SSH_MSG_UNIMPLEMENTED");
 			
 		case SSH_MSG_DISCONNECT:
-			/* TODO here */
+			/* TODO cleanup? */
 			dropbear_close("Disconnect received");
 	}
 
