@@ -1,3 +1,27 @@
+/*
+ * Dropbear - a SSH2 server
+ * 
+ * Copyright (c) 2002,2003 Matt Johnston
+ * All rights reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following condition:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE. */
+
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
@@ -5,6 +29,7 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <stdarg.h>
+#include <syslog.h>
 
 #include "options.h"
 #include "util.h"
@@ -12,52 +37,96 @@
 #include "session.h"
 #include "libtomcrypt/mycrypt.h"
 
-/* abort() on detected heap corruption */
-#define MALLOC_CHECK_ 2
+#define MAX_FMT 100
 
-void dropbear_msg(const char* format, ...) {
+static void _dropbear_log(int priority, const char* format, va_list param);
+static void _dropbear_exit(int exitcode, const char* format, va_list param);
 
-	va_list param;
+int usingsyslog = 0;
 
-	va_start(param, format);
-	fprintf(stderr, "dropbear: ");
-	vfprintf(stderr, format, param);
-	fprintf(stderr, "\n");
-	va_end(param);
+void startsyslog() {
 
+	int fd;
+
+	openlog(PROGNAME, LOG_PID, LOG_DAEMON);
+
+#ifndef DEBUG_TRACE
+	/* redirect stdin/stdout/stderr to /dev/null */
+	fd = open("/dev/null", O_RDWR);
+	if (fd != -1) {
+		dup2(fd, STDIN_FILENO);
+		dup2(fd, STDOUT_FILENO);
+		dup2(fd, STDERR_FILENO);
+		if (fd > 2) {
+			close(fd);
+		}
+	}
+#endif
+	
+	usingsyslog = 1;
 }
+
+/* the "format" string must be <= 100 characters */
 void dropbear_close(const char* format, ...) {
 
 	va_list param;
 
-#ifdef DOCLEANUP
-	session_cleanup();
-#endif
-
 	va_start(param, format);
-	fprintf(stderr, "dropbear close: ");
-	vfprintf(stderr, format, param);
-	fprintf(stderr, "\n");
+	_dropbear_exit(EXIT_SUCCESS, format, param);
 	va_end(param);
-	exit(EXIT_SUCCESS);
 
 }
 
-/* failure exit */
 void dropbear_exit(const char* format, ...) {
 
 	va_list param;
 
+	va_start(param, format);
+	_dropbear_exit(EXIT_FAILURE, format, param);
+	va_end(param);
+}
+
+/* failure exit - format must be <= 100 chars */
+static void _dropbear_exit(int exitcode, const char* format, va_list param) {
+
+#define EXIT_MESSAGE "exited: "
+#define EXIT_MESSAGE_LEN 8
+
+	char fmtbuf[MAX_FMT+EXIT_MESSAGE_LEN+1];
+
 #ifdef DOCLEANUP
 	session_cleanup();
 #endif
 
+	strcpy(fmtbuf, EXIT_MESSAGE);
+	strncat(fmtbuf, format, MAX_FMT);
+
+	_dropbear_log(LOG_DAEMON | LOG_INFO, fmtbuf, param);
+
+	exit(exitcode);
+
+}
+
+/* this is what can be called to write arbitrary log messages */
+void dropbear_log(int priority, const char* format, ...) {
+
+	va_list param;
+
 	va_start(param, format);
-	fprintf(stderr, "dropbear exit: ");
-	vfprintf(stderr, format, param);
-	fprintf(stderr, "\n");
+	_dropbear_log(priority, format, param);
 	va_end(param);
-	exit(EXIT_FAILURE);
+}
+
+/* priority is priority as with syslog() */
+static void _dropbear_log(int priority, const char* format, va_list param) {
+
+	if (usingsyslog) {
+		vsyslog(priority, format, param);
+	} else {
+		fprintf(stderr, "dropbear ");
+		vfprintf(stderr, format, param);
+		fprintf(stderr, "\n");
+	}
 
 }
 
@@ -129,7 +198,7 @@ int readln(int fd, char* buf, int count) {
 
 
 	pos = 0;
-	/* XXX hack for non-blocking*/
+	/* hack so we can block on a non-blocking fd */
 	for (;;) {
 		if (pos >= count-1) {
 			break;
@@ -226,10 +295,32 @@ int buf_readfile(buffer* buf, char* filename) {
 	close(fd);
 	return 0;
 }
+
+/* loop until the socket is closed (in case of EINTR) or
+ * we get and error.
+ * Returns 0 on fd successfully closed or bad FD, -1 otherwise */
+int m_close(int fd) {
+
+	int val;
+	do {
+		val = close(fd);
+	} while (val < 0 && errno == EINTR);
+	return (val == -1 &&
+			errno == EBADF) ? 0 : val;
+}
 	
 void * m_malloc(size_t size) {
 
-	return m_realloc(NULL, size);
+	void* ret;
+
+	if (size == 0) {
+		dropbear_exit("m_malloc failed");
+	}
+	ret = malloc(size);
+	if (ret == NULL) {
+		dropbear_exit("m_malloc failed");
+	}
+	return ret;
 
 }
 
@@ -262,6 +353,6 @@ void m_burn(void *data, unsigned int len) {
 	if (data == NULL)
 		return;
 	while (len--) {
-		*p++ = '\0';
+		*p++ = 0x66;
 	}
 }
